@@ -2,13 +2,14 @@ package Outthentic::DSL;
 
 use strict;
 
-our $VERSION = '0.0.12';
+our $VERSION = '0.1.0';
 
 use Carp;
 use Data::Dumper;
 use Outthentic::DSL::Context::Range;
 use Outthentic::DSL::Context::Default;
 use Outthentic::DSL::Context::TextBlock;
+use File::Temp qw/ tempfile /;
 
 $Data::Dumper::Terse=1;
 
@@ -296,6 +297,7 @@ sub validate {
         close $fh;
     }
 
+    my $multiline_mode;
 
     LINE: for my $l (@lines){
 
@@ -306,6 +308,13 @@ sub validate {
         next LINE unless $l =~ /\S/; # skip blank lines
 
         next LINE if $l=~ /^\s*#(.*)/; # skip comments
+        
+        if ($multiline_mode){
+            if ($l=~s/^$multiline_mode$//){
+              $multiline_mode = undef; 
+              $self->add_debug_result("multiline_mode off") if $self->{debug_mod} >= 2;
+            }
+        }
 
         if ($l=~ /^\s*begin:\s*$/) { # begin of text block
             confess "you can't switch to text block mode when within mode is enabled" 
@@ -354,16 +363,23 @@ sub validate {
 
         # validate unterminated multiline chunks
         if ($l=~/^\s*(regexp|code|generator|within|validator):\s*.*/){
-            confess "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" if defined($chunk_type);
+            confess "unterminated multiline $chunk_type found, last line: $multiline_chunk[-1]" 
+              if defined($chunk_type);
         }
 
         if ($l=~/^\s*code:\s*(.*)/){ # `code' line
 
             my $code = $1;
+
             if ($code=~s/\\\s*$//){
                  push @multiline_chunk, $code;
                  $chunk_type = 'code';
                  next LINE; # this is multiline chunk, accumulate lines until meet '\' line
+            }elsif($code=~s/<<(\S+)//){
+                $multiline_mode = $1;
+                $chunk_type = 'code';
+                $self->add_debug_result("code: multiline_mode on. marker: $multiline_mode") if $self->{debug_mod} >= 2;
+                next LINE;
             }else{
                 undef $chunk_type;
                 $self->handle_code($code);
@@ -378,6 +394,11 @@ sub validate {
                  $chunk_type = 'validator';
                  next LINE; # this is multiline chunk, accumulate lines until meet '\' line
 
+            }elsif($code=~s/<<(\S+)//){
+                $multiline_mode = $1;
+                $chunk_type = 'validator';
+                $self->add_debug_result("validator: multiline_mode on. marker: $multiline_mode") if $self->{debug_mod} >= 2;
+                next LINE;
             }else{
                 $self->handle_validator($code);
             }
@@ -391,6 +412,11 @@ sub validate {
                  $chunk_type = 'generator';
                  next LINE; # this is multiline chunk, accumulate lines until meet '\' line
 
+            }elsif($code=~s/<<(\S+)//){
+                $multiline_mode = $1;
+                $chunk_type = 'generator';
+                $self->add_debug_result("generator: multiline_mode on. marker: $multiline_mode") if $self->{debug_mod} >= 2;
+                next LINE;
             }else{
                 $self->handle_generator($code);
             }
@@ -410,7 +436,7 @@ sub validate {
 
         }elsif(defined($chunk_type)){ # multiline 
 
-            if ($l=~s/\\\s*$//) {
+             if ($l=~s/\\\s*$// or $multiline_mode ) {
 
                 push @multiline_chunk, $l;
                 next LINE; # this is multiline chunk, accumulate lines until meet '\' line
@@ -447,16 +473,50 @@ sub handle_code {
     my $code = shift;
 
     unless (ref $code){
+
         eval "package main; $code;";
         confess "eval error; sub:handle_code; code:$code\nerror: $@" if $@;
         $self->add_debug_result("handle_code OK. $code") if $self->{debug_mod} >= 3;
+
     } else {
+
         my $i = 0;
+
         my $code_to_print = join "\n", map { my $v=$_; $i++; "[$i] $v" }  @$code;
-        my $code_to_eval = join "\n", @$code;
-        eval "package main; $code_to_eval";
-        confess "eval error; sub:handle_code; code:\n$code_to_print\nerror: $@" if $@;
-        $self->add_debug_result("handle_code OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+
+        if ($code->[1]=~s/^!(.*)//){
+
+          my $ext_runner = $1;
+
+          my ($fh, $source_file) = tempfile( DIR => '/tmp' );
+
+          shift @$code;
+
+          my $code_to_eval = join "\n", @$code;
+
+          print $fh $code_to_eval;
+          close $fh;
+
+          system("$ext_runner $source_file 2>$source_file.err 1>$source_file.out");  
+
+          $self->add_debug_result("handle_external_code OK. $code_to_eval") if $self->{debug_mod} >= 3;
+          $self->add_debug_result("handle_external_code OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+
+          unless ($ENV{outth_dls_keep_ext_source_code}){
+            unlink("$source_file.out");
+            unlink("$source_file.err");
+            unlink("$source_file");
+          }
+
+        }else{
+
+          my $code_to_eval = join "\n", @$code;
+          eval "package main; $code_to_eval";
+          confess "eval error; sub:handle_code; code:\n$code_to_print\nerror: $@" if $@;
+          $self->add_debug_result("handle_code OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+
+        }
+
     }
 
 }
@@ -467,11 +527,13 @@ sub handle_validator {
     my $code = shift;
 
     unless (ref $code){
+
         my $r = eval "package main; $code;";
         confess "eval error; sub:handle_validator; code:$code\nerror: $@" if $@;
         confess "not valid return from validator, should be ARRAYREF. got: @{[ref $r]}" unless ref($r) eq 'ARRAY' ;
         $self->add_result({ status => $r->[0] , message => $r->[1] });
         $self->add_debug_result("handle_validator OK (status: $r->[0] message: $r->[1]). $code") if $self->{debug_mod} >= 2;
+
     } else {
         my $i = 0;
         my $code_to_print = join "\n", map { my $v=$_; $i++; "[$i] $v" }  @$code;
@@ -491,20 +553,59 @@ sub handle_generator {
     my $code = shift;
 
     unless (ref $code){
+
         my $arr_ref = eval "package main; $code";
         confess "eval error; sub:handle_generator; code:$code\nerror: $@" if $@;
         confess "not valid return from generator, should be ARRAYREF. got: @{[ref $arr_ref]}" unless ref($arr_ref) eq 'ARRAY' ;
         $self->add_debug_result("handle_generator OK. $code") if $self->{debug_mod} >= 3;
         $self->validate($arr_ref);
+
+
     } else {
-        my $i = 0;
-        my $code_to_print = join "\n", map { my $v=$_; $i++; "[$i] $v" }  @$code;
-        my $code_to_eval = join "\n", @$code;
-        my $arr_ref = eval " package main; $code_to_eval";
-        confess "eval error; sub:handle_generator; code:\n$code_to_print\nerror: $@" if $@;
-        confess "not valid return from generator, should be ARRAYREF. got: @{[ref $arr_ref]}" unless ref($arr_ref) eq 'ARRAY' ;
-        $self->add_debug_result("handle_generator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
-        $self->validate($arr_ref);
+
+      my $i = 0;
+      my $code_to_print = join "\n", map { my $v=$_; $i++; "[$i] $v" }  @$code;
+
+        if ($code->[0]=~s/^!(.*)//){
+  
+          my $ext_runner = $1;
+
+          my ($fh, $source_file) = tempfile( DIR => '/tmp' );
+
+          shift @$code;
+
+          my $code_to_eval = join "\n", @$code;
+
+          print $fh $code_to_eval;
+          close $fh;
+
+          system("$ext_runner $source_file 2>$source_file.err 1>$source_file.out");  
+
+          $self->add_debug_result("handle_external_generator OK. source: $source_file") if $self->{debug_mod} >= 2;
+
+          $self->add_debug_result("handle_external_generator OK. $code_to_eval") if $self->{debug_mod} >= 3;
+
+          $self->validate("$source_file.out");
+        
+          unless ($ENV{outth_dls_keep_ext_source_code}){
+            unlink("$source_file.out");
+            unlink("$source_file.err");
+            unlink("$source_file");
+          }
+
+        }else {
+
+          my $code_to_eval = join "\n", @$code;
+          my $arr_ref = eval " package main; $code_to_eval";
+
+          confess "eval error; sub:handle_generator; code:\n$code_to_print\nerror: $@" if $@;
+          confess "not valid return from generator, should be ARRAYREF. got: @{[ref $arr_ref]}" unless ref($arr_ref) eq 'ARRAY' ;
+
+          $self->add_debug_result("handle_generator OK. multiline. $code_to_eval") if $self->{debug_mod} >= 3;
+          $self->validate($arr_ref);
+  
+      }
+
     }
 
 }
@@ -1338,12 +1439,14 @@ Dsl code:
     # let's capture name and age chunks
     regexp: /(\w+)\s+(\d+)/
     
-    validator:                          \
-    my $total=0;                        \
-    for my $c (@{captures()}) {         \
-        $total+=$c->[0];                \
-    }                                   \
+    validator: <<CODE
+    my $total=0;                        
+    for my $c (@{captures()}) {         
+        $total+=$c->[0];                
+    }                                   
     [ ( $total == 72 ), "total age" ] 
+    
+    CODE
 
 
 =head1 Generators
@@ -1391,9 +1494,10 @@ Dsl code:
     # comment lines
     # and plain string check expressions:
     
-    generator:                                                  \    
-    my %d = { 'foo' => 'foo value', 'bar' => 'bar value' };     \
-    [ map  { ( "# $_", "$data{$_}" )  } keys %d ]               \
+    generator: <<CODE    
+    my %d = { 'foo' => 'foo value', 'bar' => 'bar value' };     
+    [ map  { ( "# $_", "$data{$_}" )  } keys %d ]
+    CODE
 
 New dsl code:
 
@@ -1416,16 +1520,17 @@ Input Text:
 
 Dsl code:
 
-    generator:                              \ 
-    sub next_number {                       \    
-        my $i = shift;                      \ 
-        $i++;                               \
-        return [] if $i>=5;                 \
-        [                                   \
-            'regexp: ^'.('A' x $i).'$'      \
-            "generator: next_number($i)"    \ 
+    generator:  <<CODE
+    sub next_number {                       
+        my $i = shift;                       
+        $i++;                               
+        return [] if $i>=5;                 
+        [                                   
+            'regexp: ^'.('A' x $i).'$'      
+            "generator: next_number($i)"     
         ]  
     }
+    CODE
 
 Result - verified
 
@@ -1435,24 +1540,32 @@ Result - verified
 
 =head2 Multilines in check expressions
 
-When generate and execute check expressions parser operates in a I<single line mode> :
+When parser parses check expressions it does it in a I<single line mode> :
 
 =over
 
 =item *
 
-check expressions are treated as single line strings
+a check expression is always single line string
+
 
 
 =item *
 
-input text is validated by given check expression in line by line way
+input text is parsed in line by line mode, thus every line is validated against a single line check expression
+
 
 
 =back
 
 Example.
 
+    # Input text
+    
+    Multiline
+    string
+    here
+
 Dsl code:
 
     # check list
@@ -1463,11 +1576,6 @@ Dsl code:
     string
     here
     regexp: Multiline \n string \n here
-    
-    # stdout
-    Multiline
-    string
-    here
 
 Results:
 
@@ -1480,18 +1588,37 @@ Results:
     | FAIL   | matches /Multiline \n string \n here/ |
     +--------+---------------------------------------+
 
-Use text blocks if you want multiline checks.
+Use text blocks if you want to I<represent> multiline checks.
 
 
 =head2 Multilines in perl expressions, validators and generators
 
-However when writing Perl expressions, validators or generators one could use multilines strings.
+Perl expressions, validators and generators could contain multilines expressions
+
+There are two ways to write multiline expressions:
+
+=over
+
+=item *
+
+using C<\> delimiters to split multiline string to many chunks
+
+
+
+=item *
+
+using HERE documents expressions 
+
+
+
+=back
+
+
+=head3 Back slash delimiters
 
 `\' delimiters breaks a single line text on a multi lines.
 
-Examples.
-
-Dsl code:
+Example:
 
     # What about to validate stdout
     # With sqlite database entries?
@@ -1507,6 +1634,26 @@ Dsl code:
     [ map { $_->[0] } @${results} ]
 
 
+=head3 HERE documents expressions 
+
+Is alternative to make your multiline code more readable:
+
+    # What about to validate stdout
+    # With sqlite database entries?
+    
+    generator: <<CODE
+    
+    use DBI;                                                            
+    my $dbh = DBI->connect("dbi:SQLite:dbname=t/data/test.db","","");   
+    my $sth = $dbh->prepare("SELECT name from users");                  
+    $sth->execute();                                                    
+    my $results = $sth->fetchall_arrayref;                              
+    
+    [ map { $_->[0] } @${results} ]
+    
+    CODE
+
+
 =head1 Captures
 
 Captures are pieces of data get captured when parser validate lines against a regular expressions:
@@ -1539,12 +1686,14 @@ Then captured data usually good fit for validators extra checks.
 
 Dsl code
 
-    validator:                          \
-    my $total=0;                        \
-    for my $c (@{captures()}) {         \
-        $total+=$c->[0];                \
-    }                                   \
+    validator: << CODE
+    my $total=0;                        
+    for my $c (@{captures()}) {         
+        $total+=$c->[0];                
+    }                                   
     [ ($total == 72 ), "total age of my family" ];
+    
+    CODE
 
 
 =head2 captures() function
@@ -1561,13 +1710,15 @@ Dsl code:
     
     regexp: (\d+)
     
-    validator:                          \
-    my $total=0;                        \
-    for my $c (@{captures()}) {         \
-        $total+=$c->[0];                \
-    }                                   \
+    validator:  <<CODE
+    my $total=0;                        
+    for my $c (@{captures()}) {         
+        $total+=$c->[0];                
+    }                                   
     [ ( $total > 10 ) "total amount is greater than 10" ]
     
+    CODE
+    
     
     # check if stdout contains lines
     # with date formatted as date: YYYY-MM-DD
@@ -1575,13 +1726,15 @@ Dsl code:
     
     regexp: date: (\d\d\d\d)-(\d\d)-(\d\d)
     
-    validator:                          \
-    use DateTime;                       \
-    my $c = captures()->[0];            \
-    my $dt = DateTime->new( year => $c->[0], month => $c->[1], day => $c->[2]  ); \
-    my $yesterday = DateTime->now->subtract( days =>  1 );                        \
+    validator:  <<CODE
+    use DateTime;                       
+    my $c = captures()->[0];            
+    my $dt = DateTime->new( year => $c->[0], month => $c->[1], day => $c->[2]  ); 
+    my $yesterday = DateTime->now->subtract( days =>  1 );                        
     
     [ ( DateTime->compare($dt, $yesterday) == 0 ),"first day found is - $dt and this is a yesterday" ];
+    
+    CODE
 
 
 =head2 capture() function
@@ -1808,11 +1961,13 @@ Dsl code:
     
     regexp: \d+
     
-    code:                           \
-    for my $i (@{captures()}) {     \
-        print "# ", $i->[0], "\n"   \
-    }                               \
-    print "foo/bar end"
+    code: <<CODE                           
+    for my $i (@{captures()}) {     
+        print "# ", $i->[0], "\n"   
+    }                               
+    print "# foo/bar end"
+    
+    CODE
     
     between: FOO BAR
     
@@ -1824,11 +1979,13 @@ Dsl code:
     
     regexp: \d+
     
-    code:                           \
-    for my $i (@{captures()}) {     \
-        print "#", $i->[0], "\n";   \
-    }                               \
+    code:  <<CODE
+    for my $i (@{captures()}) {     
+        print "#", $i->[0], "\n";   
+    }                               
     print "# FOO/BAR end"
+    
+    CODE
 
 Output:
 
@@ -2000,15 +2157,17 @@ Dsl code:
             regexp: \S+
         bar
     
-        code:                                   \
-            for my $s (@{stream()}) {           \
-                print "# ";                     \
-                for my $i (@{$s}){              \
-                    print $i;                   \
-                }                               \
-                print "\n";                     \
+        code:  <<CODE
+            for my $s (@{stream()}) {           
+                print "# ";                     
+                for my $i (@{$s}){              
+                    print $i;                   
+                }                               
+                print "\n";                     
             }
-        
+    
+    CODE
+    
     end:
 
 Stream function returns an arrays of I<streams>. Every stream holds all the matched lines for given I<logical block>.
@@ -2053,14 +2212,16 @@ Dsl code:
     
     regexp: \d+
     
-    code:                                   \
-        for my $s (@{stream()}) {           \
-            print "# ";                     \
-            for my $i (@{$s}){              \
-                print $i;                   \
-            }                               \
-            print "\n";                     \
+    code:  <<CODE
+        for my $s (@{stream()}) {           
+            print "# ";                     
+            for my $i (@{$s}){              
+                print $i;                   
+            }                               
+            print "\n";                     
         }
+    
+    CODE
 
 Output:
 
@@ -2069,6 +2230,55 @@ Output:
     # 0 0 0
 
 
+=head2 Inline code from other languages
+
+WARNING!!! Don't use these features in production unless this message is removed.
+
+One may use various languages in code and generators expressions. Here are examples.
+
+
+=head3 bash 
+
+    generator:  <<HERE
+    
+    !/bin/bash
+    echo OK
+    
+    HERE
+    
+    code: <<HERE
+    mkdir -p /tmp/foo   
+    HERE
+
+
+=head3 perl6
+
+    generator: <<PERL6
+    
+    !/usr/bin/perl6
+    
+    say 'OK';
+    
+    PERL6
+
+
+=head3 ruby
+
+    generator:  <<CODE
+    !/usr/bin/ruby
+    
+    puts 'OK'
+    
+    CODE
+    
+    generator:  <<<CODE
+    !ruby
+    
+    puts 'OK'
+    
+    CODE
+
+
 =head1 Author
 
 L<Aleksei Melezhik|mailto:melezhik@gmail.com>
